function Set-LinuxVM {
 
<#
    .SYNOPSIS
     
        Function to make hostname and IP configuration; install control panel,
        Hyper-V Linux integration components (LIC) and custom scripts on a Linux VM.
 
    .DESCRIPTION
     
        This function sets a new hostname and IP configuration to a Linux VM with parameters you provide.
        It changes hostname and ifconfig file of virtual machine.
        Also installs proper Linux integration components for the distribution if it's available.
        This function can also install control panels like CPanel automatically. Just use -Panel switch.
        It's possible to use your own answer file to create custom and unique setups.
             
    .PARAMETER  WhatIf
     
        Display what would happen if you would run the function with given parameters.
    
	.PARAMETER  Confirm
     
        Prompts for confirmation for each operation. Allow user to specify Yes/No to all option to stop prompting.
    
	.EXAMPLE
     
        Set-LinuxVM -Distro "Centos" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Centos.
		
	.EXAMPLE
     
        Set-LinuxVM -Distro "Suse" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -AnswerFile "C:\answer.txt" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Suse. Also applies your answer file for custom modifications.
		
	.EXAMPLE
     
        Set-LinuxVM -Distro "Centos" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -Panel "CPanel" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Centos and installs CPanel.
		
	.EXAMPLE
     
        Set-LinuxVM -Distro "Suse" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -EnableLIC "Yes" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Suse and enables integration components for this distro.
     
	.EXAMPLE
     
        Set-LinuxVM -Distro "Fedora" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -EnableLIC "Skip" -Username "root" -Password "123456"
        Sets hostname and IP addresses to VM called "Linux01" which is Fedora but skip LIC. Use Skip option if you're using a VM template which has LIC already.
 
	.EXAMPLE
     
        Set-LinuxVM -Distro "Debian" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -VMHost "hyperv01.yusufozturk.info" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Debian and runs on Hyper-V host called "hyperv01.yusufozturk.info"
		
	.EXAMPLE
     
        Set-LinuxVM -Distro "Debian" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -VMMServer "scvmm01.yusufozturk.info" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Debian and runs on SCVMM server called "scvmm01.yusufozturk.info"
		
	.EXAMPLE
     
        Set-LinuxVM -Distro "Ubuntu" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -Manager "Hyper-V" -Username "yusuf" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Ubuntu but it uses Hyper-V API instead of SCVMM.
		
	.EXAMPLE
     
        Set-LinuxVM -Distro "Ubuntu" -Hostname "server.yusufozturk.info" -IPAddress "192.168.2.2" -SubnetMask "255.255.255.0" -GatewayAddress "192.168.2.1" -PrimaryDNSAddress "8.8.8.8" -SecondaryDNSAddress "8.8.4.4" -VMName "Linux01" -Manager "Hyper-V" -VMHost "hyperv01.yusufozturk.info" -Username "yusuf" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Ubuntu but it uses Hyper-V API instead of SCVMM. Use VMHost switch if VM is on remote Hyper-V host.
	
	.EXAMPLE
     
        Set-LinuxVM -Distro "Redhat" -Hostname "server.yusufozturk.info" -IP "192.168.2.2" -Subnet "255.255.255.0" -Gateway "192.168.2.1" -DNS1 "8.8.8.8" -DNS2 "8.8.4.4" -VM "Linux01" -Username "root" -Password "123456"
        Sets hostname and IP configuration to VM called "Linux01" which is Redhat using with alias names.
		
	.INPUTS
         
        Hostname and IP configuration for Linux virtual machine.
 
    .OUTPUTS
 
        None
	
	.NOTES
         
        Author: Yusuf Ozturk
        Website: http://www.yusufozturk.info
        Email: ysfozy[at]gmail.com
        Date created: 03-May-2011
        Last modified: 08-May-2011
        Version: 2.1
 
    .LINK
         
        http://www.yusufozturk.info/category/virtual-machine-manager
 
#>
 
[CmdletBinding(SupportsShouldProcess = $true)]
param (

    # Distribution of your Linux template. Supported distributions: Centos, Fedora, Debian, Ubuntu, Redhat and Suse
	# Distribution information will be used for hostname and IP configuration.
    [Parameter(
	    Mandatory = $true,
		HelpMessage = 'Distribution of your Linux VM')]
	[ValidateSet("Centos", "Fedora", "Debian", "Ubuntu", "Redhat", "Suse")]
    [string]$Distro,
     
	# Hostname of your Linux VM. Most control panels requires fully qualified name (FQDN) for hostname.
	# Hostname value should be like this: server.yusufozturk.info
    [Parameter(
        Mandatory = $true,
        HelpMessage = 'Hostname of your Linux VM')]
    [string]$Hostname,
	
	# IP address of your Linux VM. You can assign single or multiple IPs for your Linux VM.
	# You can add multiple IP addresses with using comma between them.
	[Parameter(
        Mandatory = $true,
        HelpMessage = 'IP address of your Linux VM')]
	[Alias('IP')]
    [string]$IPAddress,

	# Subnet mask of your Linux VM.
	# A few examples of subnet mask value you can set: 255.255.255.0, 255.255.255.240 or 255.255.255.248
	[Parameter(
        Mandatory = $true,
        HelpMessage = 'Subnet mask of your Linux VM')]
	[Alias('Subnet')]
    [string]$SubnetMask,
	
	# Gateway address of your Linux VM.
	[Parameter(
        Mandatory = $true,
        HelpMessage = 'Gateway address of your Linux VM')]
	[Alias('Gateway')]
    [string]$GatewayAddress,
	
	# Primary DNS address of your Linux VM.
	[Parameter(
        Mandatory = $true,
        HelpMessage = 'Primary DNS address of your Linux VM')]
	[Alias('DNS1')]
    [string]$PrimaryDNSAddress,
	
	# Secondary DNS address of your Linux VM.
	[Parameter(
        Mandatory = $true,
        HelpMessage = 'Secondary DNS address of your Linux VM')]
	[Alias('DNS2')]
    [string]$SecondaryDNSAddress,
	
	# Name of the Linux VM you want to set.
	[Parameter(
        Mandatory = $true,
        HelpMessage = 'Name of the Linux VM')]
	[Alias('VM')]
    [string]$VMName,
	
	# Name of the Hyper-V host which currently hosts your Linux VM.
	# This parameter is not required if you have only one Linux VM with the same VM name.
	# This script prevents to modify wrong virtual machine if more than one VM exist with the same name in SCVMM.
	# You should type FQDN of Hyper-V host like hyperv01.yusufozturk.info

	[Parameter(
		Mandatory = $false,
        HelpMessage = 'Name of the Hyper-V host which currently hosts your Linux VM')]
    [string]$VMHost = 'Undefined',
	
	# Name of the SCVMM server which currently hosts your Linux VM.
	# This parameter is required if you have multiple SCVMM servers.
	# You should type FQDN of SCVMM server like scvmm01.yusufozturk.info
	# Default value: localhost
	[Parameter(
		Mandatory = $false,
        HelpMessage = 'Name of the SCVMM server which currently hosts your Linux VM')]
    [string]$VMMServer = 'localhost',
	
	# What do you use to manage your virtual machines? Valid values: "SCVMM" or "Hyper-V"
	# If you choose SCVMM, then will be used Powershell functions of SCVMM to get VM information.
	# Default value: SCVMM
	[Parameter(
		Mandatory = $false,
        HelpMessage = 'What do you use to manage your virtual machines?')]
	[ValidateSet("SCVMM","Hyper-V")]
    [string]$Manager = 'SCVMM',
	
	# Path of the answer file for custom virtual machine modifications. Example: "C:\answer.txt"
	# If you want to do additional changes on your Linux VM, add your commands into a file.
	[Parameter(
		Mandatory = $false,
        HelpMessage = 'Path of the answer file for custom VM modifications')]
    [string]$AnswerFile = 'Undefined',
	
	# Name of the control panel you want to install. Valid values: "CPanel"
	# This script allows you to install control panels like CPanel automatically.
	# This script can't install CPanel if you choose to enable LIC.
	[Parameter(
		Mandatory = $false,
        HelpMessage = 'Name of the control panel you want to install')]
	[ValidateSet("CPanel")]
    [string]$Panel = 'Undefined',
	
	# Do you want to enable Linux integration components? "Yes", "No" or "Skip"
	# Currently supported distributions: Redhat EL 5.x and Suse Enterprise 10 SP3.
	# You should choose "Skip" if your Linux VM template already has LIC.
	[Parameter(
		Mandatory = $false,
        HelpMessage = 'Do you want to enable Linux integration components? (Yes,No,Skip)')]
	[ValidateSet("Yes","No","Skip")]
    [string]$EnableLIC = 'No',
	
	# Username of the Linux virtual machine.
	# Default value: root
	[Parameter(
		Mandatory = $true,
        HelpMessage = 'Username of the Linux VM')]
    [string]$Username = 'root',
	
	# Password of the Linux virtual machine's user.
	[Parameter(
		Mandatory = $true,
        HelpMessage = 'Password of the Linux VM user')]
    [string]$Password
	
)
 
begin {

	Function Check-PSSnapin
	{
	param ($Name)
	
		$CheckSnapin = Get-PSSnapin Microsoft.SystemCenter.VirtualMachineManager -EA SilentlyContinue
		If (!$CheckSnapin)
		{
			Add-PSSnapin -Name $Name
			Write-Debug "$Name Snapin is loaded."
		}
		Else
		{
			Write-Debug "$Name Snapin is already available."
		}
	}
	
	# Determine ethernet devices
	# You can change here if you use additional ethernet devices
	
	If ($Distro -eq "Debian" -or $Distro -eq "Ubuntu")
	{
		$EmulatedDevice = "eth0"
		$SyntheticDevice = "eth0"
	}
	else
	{
		$EmulatedDevice = "eth0"
		$SyntheticDevice = "seth0"
	}
	
	# Change device name if your Linux VM template has LIC
	
	If ($EnableLIC -eq "Skip")
	{
		$EmulatedDevice = $SyntheticDevice
	}
	
	# Define Hyper-V host for WMI connections
	
	If ($VMHost -eq "Undefined")
	{
		$WMIHost = "."
	}
	else
	{
		$WMIHost = "$VMHost"
	}
	
	# Check VM count and status to prevent problems
	
	If ($Manager -eq "SCVMM")
	{
		Check-PSSnapin -Name Microsoft.SystemCenter.VirtualMachineManager
		Get-VMMServer $VMMServer
		
		If ($VMHost -eq "Undefined")
		{
			try 
			{			
				Write-Debug "Retrieving VM status..."
				$VMCount = @(Get-VM $VMName).Count
				If ($VMCount -ne "1")
				{
					Write-Warning "You have multiple VMs with same name, aborting..."
					Break;
				}
				else
				{
					$VMStatus = (Get-VM $VMName).Status
					$WMIHost = (Get-VM $VMName).HostName
					If ($VMStatus -ne "Running")
					{
						Write-Warning "VM is not running, aborting..."
						Break;
					}
					else
					{
						Write-Debug "VM state is OK!"
					}
				}
			}
			catch
			{
				Write-Error $_
				Break;
			}
		}
		else
		{
			try
			{
				Write-Debug "Retrieving VM status..."
				$VMCount = @(Get-VM $VMName -VMHost $VMHost).Count
				If ($VMCount -ne "1")
				{
					Write-Warning "You have multiple VMs with same name, aborting..."
					Break;
				}
				else
				{
					$VMStatus = (Get-VM $VMName -VMHost $VMHost).Status
					$WMIHost = $VMHost
					If ($VMStatus -ne "Running")
					{
						Write-Warning "VM is not running, aborting..."
						Break;
					}
					else
					{
						Write-Debug "VM state is OK!"
					}
				}
			}
			catch
			{
				Write-Error $_
				Break;
			}
		}			
	}
	else
	{
		try
		{
			$VMStatus = (Get-WmiObject -Computer "$WMIHost" -Namespace "root\virtualization" -Query "SELECT * FROM Msvm_ComputerSystem WHERE ElementName like '$VMName' AND caption like 'Virtual%' ").EnabledState
			If ($VMStatus -ne "2")
			{
				Write-Warning "VM is not running, aborting..."
				Break;
			}
			else
			{
				Write-Debug "VM state is OK!"
			}
		}
		catch
		{
			Write-Error $_
			Break;
		}
	}
}
	 
process {

	# Open connection to Hyper-V console
	$VMConf = Get-WmiObject -ComputerName "$WMIHost" -Namespace "root\virtualization" -Query "SELECT * FROM Msvm_ComputerSystem WHERE ElementName like '$VMName' AND caption like 'Virtual%' "
	$VSMKB=($VMConf.getRelated("Msvm_Keyboard") | select-object)
	Write-Host "Customizing Virtual Machine.."
	
   	# Ubuntu Customization
	If ($Distro -eq "Ubuntu")
	{
		$SendCommand = $VSMKB.TypeText("$Username");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("sudo su");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm /etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm /etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm /etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting hostname...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname $Hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname > /etc/hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 localhost' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 $Hostname' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting network interface...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'auto lo' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'iface lo inet loopback' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo ' ' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'auto $EmulatedDevice' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'iface $EmulatedDevice inet static' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'address $IPAddress' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'gateway $GatewayAddress' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'netmask $SubnetMask' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting dns configuration...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'search google.com' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $PrimaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $SecondaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/etc/init.d/networking restart");
		$SendCommand = $VSMKB.Typekey(0x0D)
		
		for ($a=1; $a -lt 100; $a++) 
		{
			Write-Progress -Activity "Restarting network service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
			Start-Sleep -m 50
		}

		If ($EnableLIC -eq "Yes")
		{
			$SendCommand = $VSMKB.TypeText("rm /etc/initramfs-tools/modules");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'Setting Linux IC...'");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'hv_vmbus' >>/etc/initramfs-tools/modules");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'hv_storvsc' >>/etc/initramfs-tools/modules");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'hv_blkvsc' >>/etc/initramfs-tools/modules");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'hv_netvsc' >>/etc/initramfs-tools/modules");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
		}
	}
	
	# Debian Customization
	If ($Distro -eq "Debian")
	{
		$SendCommand = $VSMKB.TypeText("$Username");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm /etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm /etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm /etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting hostname...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname $Hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname > /etc/hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 localhost' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 $Hostname' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting network interface...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'auto lo' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'iface lo inet loopback' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo ' ' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'auto $EmulatedDevice' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'iface $EmulatedDevice inet static' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'address $IPAddress' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'gateway $GatewayAddress' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'netmask $SubnetMask' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '' >>/etc/network/interfaces");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting dns configuration...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'search google.com' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $PrimaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $SecondaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/etc/init.d/networking restart");
		$SendCommand = $VSMKB.Typekey(0x0D)

		for ($a=1; $a -lt 100; $a++) 
		{
			Write-Progress -Activity "Restarting network service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
			Start-Sleep -m 50
		}
		
		If ($EnableLIC -eq "Yes")
		{
			Write-Warning "** Debian Custom Kernel Upgrade **"
			Write-Warning "Warning! This is not an official solution and should be used for testing purposes."
			Write-Warning "We don't recommend you to use this in production servers."
			Write-Warning "After this upgrade, you may not get support from Debian anymore."
			Write-Warning "You can cancel this installation in 5 seconds by hitting CTRL+C."
			Start-Sleep 5
			$SendCommand = $VSMKB.TypeText("wget -c http://www.yusufozturk.info/linux/linux-image-2.6.36-hyperv-debian.x86_64.deb");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading kernel image..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 2400
			}
			$SendCommand = $VSMKB.TypeText("wget -c http://www.yusufozturk.info/linux/linux-headers-2.6.36-hyperv-debian.x86_64.deb");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading kernel headers..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 2400
			}
			$SendCommand = $VSMKB.TypeText("dpkg -i linux-image-2.6.36-hyperv-debian.x86_64.deb linux-headers-2.6.36-hyperv-debian.x86_64.deb");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing new kernel ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("echo -e 'hv_vmbus\nhv_storvsc\nhv_blkvsc\nhv_netvsc' >> /etc/initramfs-tools/modules");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 5
			$SendCommand = $VSMKB.TypeText("update-initramfs -u -k 2.6.36-hyperv");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Updating initramfs ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
		}
	}
	
	# Centos Customization
	If ($Distro -eq "Centos")
	{
		$SendCommand = $VSMKB.TypeText("$Username");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting hostname...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname $Hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname > /etc/hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 localhost' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 $Hostname' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting network interface...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'DEVICE=$EmulatedDevice' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'ONBOOT=yes' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETWORKING=yes' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETWORKING_IPV6=no' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'HOSTNAME=$Hostname' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting dns configuration...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'search google.com' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $PrimaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $SecondaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("service network restart");
		$SendCommand = $VSMKB.Typekey(0x0D)
		
		for ($a=1; $a -lt 100; $a++) 
		{
			Write-Progress -Activity "Restarting network service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
			Start-Sleep -m 50
		}
		
		If ($EnableLIC -eq "Yes")
		{
			$SendCommand = $VSMKB.TypeText("cd /home");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("wget http://www.yusufozturk.info/linux/tools/LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("tar -xvf LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 5
			$SendCommand = $VSMKB.TypeText("cd LinuxICv21");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("rpm -ivh adjtimex-1.20-2.1.x86_64.rpm");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing adtimex service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}			
			$SendCommand = $VSMKB.TypeText("make");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Making ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}
			$SendCommand = $VSMKB.TypeText("make install");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("rm -vf /etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'DEVICE=$SyntheticDevice' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'ONBOOT=yes' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
		}
	}
	
	# Fedora Customization
	If ($Distro -eq "Fedora")
	{
		$SendCommand = $VSMKB.TypeText("$Username");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting hostname...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname $Hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname > /etc/hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 localhost' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 $Hostname' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting network interface...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'DEVICE=$EmulatedDevice' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'ONBOOT=yes' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETWORKING=yes' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETWORKING_IPV6=no' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'HOSTNAME=$Hostname' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting dns configuration...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'search google.com' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $PrimaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $SecondaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("service network restart");
		$SendCommand = $VSMKB.Typekey(0x0D)
		for ($a=1; $a -lt 100; $a++) 
		{
			Write-Progress -Activity "Restarting network service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
			Start-Sleep -m 50
		}
		
		If ($EnableLIC -eq "Yes")
		{
			$SendCommand = $VSMKB.TypeText("cd /home");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("wget http://www.yusufozturk.info/linux/tools/LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("tar -xvf LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 5
			$SendCommand = $VSMKB.TypeText("cd LinuxICv21");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("rpm -ivh adjtimex-1.20-2.1.x86_64.rpm");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing adtimex service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}				
			$SendCommand = $VSMKB.TypeText("make");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Making ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}
			$SendCommand = $VSMKB.TypeText("make install");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("rm -vf /etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'DEVICE=$SyntheticDevice' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'ONBOOT=yes' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
		}
	}
	
	# Redhat Customization
	If ($Distro -eq "Redhat")
	{
		$SendCommand = $VSMKB.TypeText("$Username");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting hostname...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname $Hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname > /etc/hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 localhost' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 $Hostname' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting network interface...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'DEVICE=$EmulatedDevice' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'ONBOOT=yes' >>/etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETWORKING=yes' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETWORKING_IPV6=no' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'HOSTNAME=$Hostname' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting dns configuration...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'search google.com' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $PrimaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $SecondaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("service network restart");
		$SendCommand = $VSMKB.Typekey(0x0D)
		for ($a=1; $a -lt 100; $a++) 
		{
			Write-Progress -Activity "Restarting network service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
			Start-Sleep -m 50
		}
		
		If ($EnableLIC -eq "Yes")
		{
			$SendCommand = $VSMKB.TypeText("cd /home");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("wget http://www.yusufozturk.info/linux/tools/LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("tar -xvf LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 5
			$SendCommand = $VSMKB.TypeText("cd LinuxICv21");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("rpm -ivh adjtimex-1.20-2.1.x86_64.rpm");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing adtimex service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}				
			$SendCommand = $VSMKB.TypeText("make");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Making ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}
			$SendCommand = $VSMKB.TypeText("make install");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("rm -vf /etc/sysconfig/network-scripts/ifcfg-$EmulatedDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'DEVICE=$SyntheticDevice' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'ONBOOT=yes' >>/etc/sysconfig/network-scripts/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
		}
	}
	
	# SUSE and OpenSUSE Customization
	If ($Distro -eq "Suse")
	{
		$SendCommand = $VSMKB.TypeText("$Username");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("$Password");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/HOSTNAME");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network/routes");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting hostname...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname $Hostname");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("/bin/hostname > /etc/HOSTNAME");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 localhost' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo '127.0.0.1 $Hostname' >>/etc/hosts");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting network interface...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'STARTMODE=auto' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'FIREWALL=no' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'USERCONTROL=no' >>/etc/sysconfig/network/ifcfg-$EmulatedDevice");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'default $GatewayAddress' >>/etc/sysconfig/network/routes");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'Setting dns configuration...'");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'search google.com' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $PrimaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("echo 'nameserver $SecondaryDNSAddress' >>/etc/resolv.conf");
		$SendCommand = $VSMKB.Typekey(0x0D)
		Start-Sleep 3
		$SendCommand = $VSMKB.TypeText("service network restart");
		$SendCommand = $VSMKB.Typekey(0x0D)
		for ($a=1; $a -lt 100; $a++) 
		{
			Write-Progress -Activity "Restarting network service ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
			Start-Sleep -m 50
		}
		
		If ($EnableLIC -eq "Yes")
		{
			$SendCommand = $VSMKB.TypeText("cd /home");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("wget http://www.yusufozturk.info/linux/tools/LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("tar -xvf LinuxICv21.tar");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 5
			$SendCommand = $VSMKB.TypeText("cd LinuxICv21");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3			
			$SendCommand = $VSMKB.TypeText("make");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Making ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}
			$SendCommand = $VSMKB.TypeText("make install");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Installing Linux IC ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 600
			}
			$SendCommand = $VSMKB.TypeText("rm -f /etc/sysconfig/network/ifcfg-$EmulatedDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'BOOTPROTO=static' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'IPADDR=$IPAddress' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'NETMASK=$SubnetMask' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'GATEWAY=$GatewayAddress' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'STARTMODE=auto' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'FIREWALL=no' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("echo 'USERCONTROL=no' >>/etc/sysconfig/network/ifcfg-$SyntheticDevice");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
		}
	}
	
	# Custom Script Support
	If ($AnswerFile -eq "Undefined")
	{
		Write-Debug "No answer file provided!"
	}
	else
	{
		if (!(Test-Path -Path $AnswerFile)) 
		{
            Write-Warning "Could not find answer file!"
        }
		else
		{
			$Contents = Get-Content $AnswerFile
			Foreach ($Content in $Contents)
			{
				$SendCommand = $VSMKB.TypeText("$Content");
				$SendCommand = $VSMKB.Typekey(0x0D)
				Start-Sleep 30
			}
		}
	}
	
	# Installing CPanel for Redhat and Centos
	If ($EnableLIC -eq "Skip")
	{
		If ($Distro -eq "Centos" -or $Distro -eq "Redhat")
		{
			Write-Warning "Starting CPanel Installation.."
			$SendCommand = $VSMKB.TypeText("cd /home");
			$SendCommand = $VSMKB.Typekey(0x0D)
			Start-Sleep 3
			$SendCommand = $VSMKB.TypeText("wget -N http://httpupdate.cpanel.net/latest");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Downloading CPanel setup file ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 300
			}
			$SendCommand = $VSMKB.TypeText("sh latest");
			$SendCommand = $VSMKB.Typekey(0x0D)
			for ($a=1; $a -lt 100; $a++) 
			{
				Write-Progress -Activity "Starting CPanel installation ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
				Start-Sleep -m 50
			}
		}
	}
	
	# Close Serial Port Connection
	$SendCommand = $VSMKB.TypeText("exit");
	$SendCommand = $VSMKB.Typekey(0x0D)
	
	# Ethernet Configuration
	# If you choose to enable LIC, then this script follows:
		# 1. Stop Virtual Machine
		# 2. Get MAC address and VLAN ID of Virtual Machine
		# 3. Remove Emulated Ethernet Device
		# 4. Add new Synthetic Ethernet Device
		# 5. Set same MAC address and VLAN ID to the new device
		# 6. Start Virtual Machine
	# If you assign new MAC address, then eth0 name may change to eth1.
	
	If ($EnableLIC -eq "Yes")
	{
		If ($Manager -eq "SCVMM")
		{
			try
			{
				If ($VMHost -eq "Undefined")
				{
					Stop-VM $VMName
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Stopping virtual machine ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}
					$NetworkAdapter = (Get-VM $VMName | Get-VirtualNetworkAdapter)
					$VirtualNetwork = $NetworkAdapter.VirtualNetwork
					$EthernetMAC = $NetworkAdapter.EthernetAddress
					$EthernetVLAN = $NetworkAdapter.VLanId
					Get-VM $VMName | Get-VirtualNetworkAdapter | Remove-VirtualNetworkAdapter
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Removing virtual network adapter ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}	
					Get-VM $VMName | New-VirtualNetworkAdapter -VirtualNetwork "$VirtualNetwork" -VLanEnabled $True -VLanId "$EthernetVLAN" -PhysicalAddress "$EthernetMAC" -Synthetic
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Setting new virtual network adapter ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}
					Start-VM $VMName					
				}
				else
				{
					Stop-VM $VMName -VMHost $VMHost
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Stopping virtual machine ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}
					$NetworkAdapter = (Get-VM $VMName -VMHost $VMHost | Get-VirtualNetworkAdapter)
					$VirtualNetwork = $NetworkAdapter.VirtualNetwork
					$EthernetMAC = $NetworkAdapter.EthernetAddress
					$EthernetVLAN = $NetworkAdapter.VLanId
					Get-VM $VMName -VMHost $VMHost | Get-VirtualNetworkAdapter | Remove-VirtualNetworkAdapter
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Removing virtual network adapter ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}	
					Get-VM $VMName -VMHost $VMHost | New-VirtualNetworkAdapter -VirtualNetwork "$VirtualNetwork" -VLanEnabled $True -VLanId "$EthernetVLAN" -PhysicalAddress "$EthernetMAC" -Synthetic
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Setting new virtual network adapter ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}
					Start-VM $VMName -VMHost $VMHost
				}	
			}
			catch
			{
				Write-Error $_
				Break;
			}
		}		
		else
		{
			try
			{
				# Get Emulated Virtual Network Adapter and Remove
				$VMConf = Get-WmiObject -Computer "$WMIHost" -Namespace "root\virtualization" -Query "SELECT * FROM Msvm_ComputerSystem WHERE ElementName like '$VMName' AND caption like 'Virtual%' "
				$VMSetting = Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "associators of {$($VMConf.__Path)} where resultclass=MSVM_VirtualSystemSettingData" | where-object {$_.instanceID -eq "Microsoft:$($VMConf.name)"}
				$VMConf.RequestStateChange(3)
				for ($a=1; $a -lt 100; $a++) 
				{
					Write-Progress -Activity "Stopping virtual machine ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
					Start-Sleep -m 50
				}
				$NetworkAdapter = Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "associators of {$($VMSetting.__Path)} where resultclass=MsVM_EmulatedEthernetPortSettingData"
				If ($NetworkAdapter.Caption -eq $null)
				{
					Write-Warning "VM has no emulated adapter, searching for synthetic adapters.."
					$NetworkAdapter = Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "associators of {$($VMSetting.__Path)} where resultclass=MsVM_SyntheticEthernetPortSettingData"	
					If ($NetworkAdapter.Caption -eq $null)
					{
						Write-Debug "No network adapters detected."
						$VMConf.RequestStateChange(1)
						$NICStatus = "1"
					}
					else
					{
						Write-Debug "VM already has a synthetic adapter."
						$VMConf.RequestStateChange(1)
						$NICStatus = "2"
					}
				}
				else
				{
					$VirtualNetwork = Get-WmiObject -ComputerName $VMConf.__Server -NameSpace "root\virtualization" -Query "associators OF {$($NetworkAdapter.Connection[0])} where resultclass = Msvm_VirtualSwitch"
					$VirtualLAN = Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "associators of {$($NetworkAdapter.Connection[0])} where assocClass=msvm_bindsto" | foreach-object {Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "associators of {$_} where assocClass=MSVM_NetWorkElementSettingData"}
					$VirtualNetworkName = $VirtualNetwork.ElementName
					$EthernetMAC = $NetworkAdapter.address
					$EthernetVLAN = $VirtualLAN.accessVlan
					$VSMgtSvc = Get-WmiObject -ComputerName $VMSetting.__Server -Namespace "root\virtualization" -Class "MSVM_VirtualSystemManagementService"
					$VSMgtSvc.RemoveVirtualSystemResources($VMConf.__Path, @( $NetworkAdapter.__Path ))
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Removing virtual network adapter ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}
					$NICStatus = "3"
				}
				
				# Create New Synthetic Network Adapter
				If ($NICStatus -eq "1")
				{
					$VMConf.RequestStateChange(2)
				}
				elseif ($NICStatus -eq "2")
				{
					$VMConf.RequestStateChange(2)
				}
				else
				{
					[String]$GUID=("{"+[System.GUID]::NewGUID().ToString()+"}")
					[String]$Name=([System.GUID]::NewGUID().ToString())
					$VMConf = Get-WmiObject -ComputerName "$WMIHost" -Namespace "root\virtualization" -Query "SELECT * FROM Msvm_ComputerSystem WHERE ElementName like '$VMName' AND caption like 'Virtual%' "
					$VMSetting = Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "associators of {$($VMConf.__Path)} where resultclass=MSVM_VirtualSystemSettingData" | where-object {$_.instanceID -eq "Microsoft:$($VMConf.name)"}
					$NetworkAllocation = ((Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "SELECT * FROM MSVM_AllocationCapabilities WHERE ResourceType = '10' AND ResourceSubType = 'Microsoft Synthetic Ethernet Port'").__Path).Replace("\", "\\")
					$NicRASD = New-Object System.Management.ManagementObject((Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "SELECT * FROM MSVM_SettingsDefineCapabilities WHERE ValueRange=0 AND GroupComponent = '$NetworkAllocation'").PartComponent)
					$NicRASD.VirtualSystemIdentifiers=@($GUID)
					$NicRASD.ElementName = "Network Adapter"
					$NicRASD.address = $EthernetMAC
					$VirtualSwitch = Get-WmiObject -ComputerName $VMConf.__Server -Namespace "root\virtualization" -Query "Select * From MsVM_VirtualSwitch Where elementname like '$VirtualNetworkName' "
					$SwitchMgtSvc=(Get-WmiObject -ComputerName $VirtualSwitch.__Server -NameSpace "root\virtualization" -Query "Select * From MsVM_VirtualSwitchManagementService")
					$Result = $SwitchMgtSvc.CreateSwitchPort($VirtualSwitch.__Path, $Name, $Name) 
					$NicRASD.connection = @($Result.CreatedSwitchPort)
					$VSMgtSvc = Get-WmiObject -ComputerName $NicRASD.__Server -Namespace "root\virtualization" -Class "MSVM_VirtualSystemManagementService"
					$VSMgtSvc.AddVirtualSystemResources($VMConf.__Path, @($NicRASD.GetText([System.Management.TextFormat]::WmiDtd20)))
					$vlanSetting = Get-WmiObject -ComputerName $NicRASD.__Server -Namespace "root\virtualization" -Query "associators of {$($NicRASD.connection[0])} where assocClass=msvm_bindsto" | foreach-object {Get-WmiObject -ComputerName $NicRASD.__Server -Namespace "root\virtualization" -Query "associators of {$_} where assocClass=MSVM_NetWorkElementSettingData"}
					$vlanSetting.accessVlan = $EthernetVLAN
					[wmi]$vlanSetting.put().path
					for ($a=1; $a -lt 100; $a++) 
					{
						Write-Progress -Activity "Setting new virtual network adapter ..." -PercentComplete $a -CurrentOperation "$a% complete" -Status "Please wait."
						Start-Sleep -m 50
					}	
					$VMConf.RequestStateChange(2)
				}
			}
			catch
			{
				Write-Error $_
				Break;
			}
		}
	}	
}
 
end {
     
	 Write-Host Done!
}
} # End of function Set-LinuxVM

Export-ModuleMember Set-LinuxVM